import { EOrientation } from '../types';

export const PORTRAIT_HEIGHT = 971.33
export const PORTRAIT_WIDTH = 680.3

export const LANDSCAPE_HEIGHT = 718.1
export const LANDSCAPE_WIDTH = 1046.9

export class Page {

  private orientation: EOrientation = EOrientation.portrait
  private fullHeight = 0 // размер страницы
  private balanceHeight = 0 // кол-во свободной высоты
  private currentContentHeight = 0

  private elems: HTMLDivElement[] = []
  private template: string

  constructor(orientation: EOrientation, template: string) {
    this.orientation = orientation
    if (orientation == EOrientation.portrait){
      this.balanceHeight = PORTRAIT_HEIGHT
      this.fullHeight = PORTRAIT_HEIGHT
    }
    if (orientation == EOrientation.landscape){
      this.balanceHeight = LANDSCAPE_HEIGHT
      this.fullHeight = PORTRAIT_HEIGHT
    }

    if (!template.includes('{{content}}'))
      throw new Error('Неверный шаблон старницы')

    this.template = template
    return this
  }

  get pageContent() {
    return this.template.replace(
      `{{content}}`,
      this.elems.reduce((accum, elem) => accum + elem.outerHTML, '')
    )
  }
  get height() {
    return {
      fullHeight: this.fullHeight,
      balanceHeight: this.balanceHeight,
      currentContentHeight: this.currentContentHeight
    }
  }

  insertElem(elem: HTMLDivElement) {
    if (elem.offsetHeight < this.balanceHeight) {
      this.elems?.push(elem.cloneNode(true) as HTMLDivElement)
      this.balanceHeight = this.balanceHeight - elem.offsetHeight
      this.currentContentHeight = this.currentContentHeight + elem.offsetHeight
      return this
    }
    return null
  }
}