import { EOrientation } from '../types'
import { TableShrink } from "./TableShrink"
import { TableTransfrom } from "./TableTransfrom"
import { TableUtils } from "./TableUtils"


export class TableElem {
  private _elem: HTMLDivElement
  private _height: number
  private _width: number

  private _tableWrapper: HTMLDivElement
  private _tHead: HTMLDivElement
  private _tBodyRows: HTMLDivElement[] = []


  constructor(elem: HTMLDivElement, orientation: EOrientation) {

    const tableDiv = TableTransfrom.transform(elem)

    TableShrink.shrink(tableDiv, orientation)

    this._elem = tableDiv

    this._height = tableDiv.offsetHeight
    this._width = tableDiv.offsetWidth

    this._tableWrapper = tableDiv.querySelector('.table')! as HTMLDivElement
    this._tHead = tableDiv.querySelector('.table-header')!
    this._tBodyRows = [...tableDiv.querySelectorAll('.table-body .table-row')] as HTMLDivElement[]

  }

  get isEmpty() {
    return !this._tBodyRows.length
  }
  get minHeght() {
    return this._tHead.offsetHeight + this._tBodyRows[0].offsetHeight
  }

  getTable(balanceHeight: number): HTMLDivElement {

    const tableContainer = document.createElement('div') as HTMLDivElement
    const tableWrapper = this._tableWrapper.cloneNode()
    const tableHeader = this._tHead.cloneNode(true)

    const tableBody = TableUtils.createTableElement('table-body')

    tableContainer.appendChild(tableWrapper)
    tableWrapper.appendChild(tableHeader)
    tableWrapper.appendChild(tableBody)

    // Вставляем чтобы наш элемент получил высоту
    const iframe = document.querySelector('.iframe-hidden-dom') as HTMLIFrameElement
    const placeForInsert = iframe.contentDocument?.body.querySelector('.hidden-dom .template-constructor')
    placeForInsert?.appendChild(tableContainer)

    let stoplose = false
    this._tBodyRows = this._tBodyRows.reduce((accum, tableRow) => {
      if (tableRow.offsetHeight + tableContainer.offsetHeight < balanceHeight && !stoplose) {
        tableBody.appendChild(tableRow.cloneNode(true))
        return accum
      }
      stoplose = true
      return [...accum, tableRow]
    }, [] as HTMLDivElement[])

    return tableContainer
  }
}