import { EOrientation } from '../types'
import { MIN_WIDTH_CELL_ENTITY } from "./constant"
import { LANDSCAPE_WIDTH, PORTRAIT_WIDTH } from './Page'
import { TableUtils } from "./TableUtils"
import { CellEntity } from "./type"


// Изменяет таблицу по размеру страницы с учетом всех ширин
export class TableShrink {
  static shrink(divTable: HTMLDivElement, orientation: EOrientation) {

    const widthColls = this.calculateDynamicTableWidth(divTable, orientation)

    // Если ширина суммы всех ячек больше страницы, то делаем scale
    // При этом помним, метод calculateDynamicTableWidth пытался уместить все ячейки в размер одной страницы 
    const maxWidthPage = orientation == EOrientation.portrait ? PORTRAIT_WIDTH : LANDSCAPE_WIDTH
    const sumWidthColls = widthColls.reduce((sum, width) => sum + width, 0)

    const scale = maxWidthPage / sumWidthColls

    const divRows = TableUtils.getTableElement(divTable, '.table-row') as HTMLDivElement[]

    divRows.forEach(row => {
      const divCells = TableUtils.getTableElement(row, '.table-cell') as HTMLDivElement[]

      divCells.forEach((cell, index) => {

        cell.style.minHeight = '18pt'

        if (scale < 1) {
          
          if (scale <= 0.5) {
            cell.style.lineHeight = `${20.72 * (0.7)}pt`
            cell.style.fontSize = `${14 * (0.7)}pt`
          }
          if (scale > 0.5 && scale < 0.7) {
            cell.style.lineHeight = `${20.72 * 0.85}pt`
            cell.style.fontSize = `${14 * 0.85}pt`
          }

          const img = cell.querySelector('img')
          if (img) {
            img.style.width = `${100 * (scale)}px`
            img.style.maxWidth = `${100 * (scale)}px`
            img.style.minWidth = `${100 * (scale)}px`
          }
        }

        cell.style.width = `${widthColls[index] * scale - 5}px`
        cell.style.maxWidth = `${widthColls[index] * scale - 5}px`
        cell.style.minWidth = `${widthColls[index] * scale - 5}px`
      })
    })
  }

  // Пытаемся распределить все ячейки таблицы в размер старницы
  static calculateDynamicTableWidth(divTable: HTMLDivElement, orientation: EOrientation) {
    //Получаем размер всея ячеек таблицы до сжатия
    //То есть сейчас они все в одну строку
    const [collEntity, widthColl] = this.getWidthTable(divTable)

    const maxWidthPage = orientation == EOrientation.portrait ? PORTRAIT_WIDTH : LANDSCAPE_WIDTH

    const scale = maxWidthPage / widthColl.reduce((sum, width) => sum + width, 0)

    // Место, которое мы отдали другим ячейкам, чтобы они были хотя бы шириной в MIN_WIDTH_CELL_ENTITY 
    let distributeWidth = 0
    // Если ячейка имеет свободное место для сжатия то записываем ее
    const freeWidth: number[] = new Array(widthColl.length).fill(0)

    let tempWidth = widthColl
      .map(width => width * scale)
      .map((width, index) => {
        const minWidth = MIN_WIDTH_CELL_ENTITY[collEntity[index]]
        if (width <= minWidth) {
          distributeWidth += minWidth - width
          return minWidth
        }

        if (width > minWidth)
          freeWidth[index] = width - minWidth

        return width
      })

    // Коэффициент распределения свободного места
    // Для того чтобы все ячейки имеющие свободно место отдавали не все свое место, а только равнозначную часть
    // Допустим свободное место [50,20], а нужно распределить 30, то вместо того чтобы забрать у первой ячейки все свободное место, мы забираем у всех по-немногу 
    const coeffDistribution = distributeWidth / freeWidth.reduce((acc, width) => acc + width, 0)

    const resWidth = tempWidth.map((width, index) => {
      if (freeWidth[index] > 0 && coeffDistribution < 1) {
        distributeWidth -= freeWidth[index] * coeffDistribution
        return width - freeWidth[index] * coeffDistribution
      }
      return width
    })

    return resWidth
  }

  static getWidthTable(divTable: HTMLDivElement): [CellEntity[], number[]] {

    const divRows = TableUtils.getTableElement(divTable, '.table-row') as HTMLDivElement[]

    const countColl = TableUtils.getTableElement(divRows[0], '.table-cell').length

    const maxWidth: number[] = Array(countColl).fill(0)
    const collEntity: CellEntity[] = Array(countColl).fill('')

    divRows.forEach(row => {
      const divCells = TableUtils.getTableElement(row, '.table-cell') as HTMLDivElement[]

      divCells.forEach((cell, index) => {
        collEntity[index] = TableUtils.getCellType(cell)
        maxWidth[index] = Math.max(maxWidth[index] ?? 0, cell.offsetWidth)
      })
    })

    return [collEntity, maxWidth]
  }
}