import { TableUtils } from "./TableUtils"


//Трансформирует таблицу в div вид
export class TableTransfrom {

  // Трансформирует таблицу из table тега, в таблицу из div
  static transform(elem: HTMLDivElement) {

    // Создаем строку для шаблона
    const divTemplateRow = document.createElement('div')

    // Создаем таблицу со всеми атрибутами
    const divTable = elem.querySelector('.table')!.cloneNode() as HTMLDivElement
    divTemplateRow.appendChild(divTable)

    // Вставляем в наш скрытый дом, для вычисления ширины в будущем
    const iframe = document.querySelector('.iframe-hidden-dom') as HTMLIFrameElement
    iframe.contentDocument?.body.querySelector('.hidden-dom .template-constructor')?.appendChild(divTemplateRow)

    // Транформируем нашу таблицу из табличного вида в div вид
    const divHeader = this.transformHeader(elem.querySelector('thead') as HTMLTableSectionElement)
    const divBody = this.transformBody(elem.querySelector('tbody') as HTMLTableSectionElement)

    divTable.appendChild(divHeader)
    divTable.appendChild(divBody)

    return divTemplateRow
  }

 

  static transformRow(row: HTMLTableRowElement) {
    const tableCells = row.cells;
    const divRow = TableUtils.createTableElement('table-row');
    [...tableCells].forEach((tableCell) => {

      const divCell = TableUtils.createTableElement('table-cell')
      tableCell.childNodes.forEach(cellChildNode => divCell.appendChild(cellChildNode.cloneNode(true)))

      divRow.appendChild(divCell)
    })

    return divRow
  }

  static transformHeader(tableHeader: HTMLTableSectionElement) {

    const tableRow = tableHeader.querySelector('tr') as HTMLTableRowElement

    const divHeader = TableUtils.createTableElement('table-header')
    const divRow = this.transformRow(tableRow)

    divHeader.appendChild(divRow)

    return divHeader
  }

  static transformBody(tableBody: HTMLTableSectionElement) {
    const tableRows = [...tableBody.querySelectorAll('tr')];

    const divBody = TableUtils.createTableElement('table-body')
    const divRows = tableRows.map((e) => this.transformRow(e))

    divRows.forEach((e) => divBody.appendChild(e))

    return divBody
  }
}